///////////////////////////////////////////////////////////////////////////////////////////////////////
// Module : DemoX20.c
// Function : Generates interrupts and displays the detected interrupts-per-second occurring on a DIO.
///////////////////////////////////////////////////////////////////////////////////////////////////////
#include <windows.h>
#include <stdio.h>
#include <conio.h>

#define USING_LOADLIBRARY_METHOD
#include "sx20func.h"

// SX20.DLL Startup and Shutdown functions:
unsigned long SX20_DLLOpen(void);   // Open DLL and initialize function pointers.
void SX20_DLLClose(void);  // Close DLL.

// Specify desired number of interrupts per second.
#define INTS_PER_SEC  5

// Specify number of boards in the system.
#define NUM_BOARDS    1

// DIO channel to be monitored.  Set to any channel from 0 to 63.
#define DIO_CHAN    0

// Interrupt counters, one per board.
static volatile DWORD IntCounts[NUM_BOARDS] = { 0, };


//////////////////////////////////////////////////////////////////////
//  ISR callback function, executed in context of interrupt thread.

void AppISR(DWORD index )
{
	// If DIO has pending interrupt request (which it should, since this is executing) ...
	if ( SX20_ChanCapGet( index, DIO_CHAN ) )
	{
		IntCounts[index]++;             // Increment DIO's interrupt counter.
		SX20_ChanArmSet( index, DIO_CHAN, FALSE );  // Clear the interrupt request.
		SX20_ChanArmSet( index, DIO_CHAN, TRUE );   // Enable the next capture.
	}
	// Unmask board's master interrupt.
	SX20_BoardIenSet( index, TRUE );
}

//////////////////////////////////////////////////////////////////////
// Open board.  Returns error code if failed, else zero.

static DWORD board_open( DWORD index )
{
	DWORD errcode;
	
	// Declare board to driver and launch the interrupt thread.  We assume only one X20 board
	// is in the system.
 	SX20_BoardOpen( index, index, AppISR );
	
	// List errors, if any.
	errcode = SX20_ErrFlagsGet( index );
	if ( errcode )
	{
		printf( "ONE OR MORE ERRORS DETECTED ON BOARD %d:\n", index );
		if ( errcode & ERR_KERNEL_DRIVER  ) printf( " * Failed to open kernel-mode driver\n" );
		if ( errcode & ERR_DEVICE_NOT_FOUND ) printf( " * Can't detect/register board\n" );
		if ( errcode & ERR_INTERRUPT    ) printf( " * Can't enable IRQ\n" );
		if ( errcode & ERR_THREAD     ) printf( " * Can't launch interrupt thread\n" );
		if ( errcode & ERR_VERSION      ) printf( " * Driver version is not supported\n" );
	}
	
	// Indicate whether successful.
	return errcode;
}

//////////////////////////////////////////////////////////////////////
// Application main function.

VOID __cdecl main()
{
	DWORD i;
	DWORD j;
	DWORD nboards = 0;
	
	printf( "\n\nSENSORAY X20 DEMO -- HIT ANY KEY TO TERMINATE\n\n" );
	
	// Link to SX20.DLL.
	if ( SX20_DLLOpen() )
	{
		printf( "Error: failed to open SX20.DLL\n" );
		while ( !kbhit() );
		return;
	}
	
	// Declare boards to driver and launch the interrupt threads.
	for ( nboards = 0; nboards < NUM_BOARDS; nboards++ )
	{
		if ( board_open( nboards ) )
			break;
	}
	
	// If all boards were successfully opened ...
	if ( nboards == NUM_BOARDS )
	{
		// For each board ...
		for ( i = 0; i < NUM_BOARDS; i++ )
		{
			// Configure target DIO's 8-channel group as outputs.
			SX20_OctetDdrSet( i, DIO_CHAN >> 3, 1 );
			// Enable event capturing and interrupts on target DIO channel.
			SX20_ChanArmSet( i, DIO_CHAN, TRUE );
			SX20_ChanIenSet( i, DIO_CHAN, TRUE );
			// Enable board master interrupt.
			SX20_BoardIenSet( i, TRUE );
		}
		
		// Repeat until a key is pressed ...
		do
		{
			// Generate the required number of interrupts (burst) for each board.
			for ( j = 0; j < INTS_PER_SEC; j++ )
			{
				for ( i = 0; i < NUM_BOARDS; i++ )
				{ 
					Sleep(1);
					SX20_ChanDioSet(i, DIO_CHAN, 1);
					Sleep(1);
					SX20_ChanDioSet(i, DIO_CHAN, 0);
				}
			}
			
			// Suspend main thread for one second.
			Sleep( 1000 );
			
			// Display and reset interrupt counts for each board
			for ( i = 0; i < NUM_BOARDS; i++ )
			{
				printf( "Board %d ints/sec: %d\n", i, IntCounts[i] );
				IntCounts[i] = 0;
			}
		}
		while ( !kbhit() );
	}
	

	// Close all open boards.
	for ( i = 0; i < nboards; i++ )
		SX20_BoardClose( i );
	
	// Unlink from SX20.DLL.
	SX20_DLLClose();
}
